@@ -18,10 +18,11 @@ import threading
 import time
 import os
 
-import pg  # the module under test
-
+from collections import namedtuple
 from decimal import Decimal
 
+import pg  # the module under test
+
 # We need a database to test against.  If LOCAL_PyGreSQL.py exists we will
 # get our information from that.  Otherwise we use the defaults.
 # These tests should be run with various PostgreSQL versions and databases
@@ -138,9 +139,12 @@ class TestConnectObject(unittest.TestCase):
 
     @unittest.skipIf(do_not_ask_for_host, do_not_ask_for_host_reason)
     def testAttributeHost(self):
-        def_host = 'localhost'
+        if dbhost and not dbhost.startswith('/'):
+            host = dbhost
+        else:
+            host = 'localhost'
         self.assertIsInstance(self.connection.host, str)
-        self.assertEqual(self.connection.host, dbhost or def_host)
+        self.assertEqual(self.connection.host, host)
 
     def testAttributeOptions(self):
         no_options = ''
@@ -159,7 +163,7 @@ class TestConnectObject(unittest.TestCase):
     def testAttributeServerVersion(self):
         server_version = self.connection.server_version
         self.assertIsInstance(server_version, int)
-        self.assertTrue(70400 <= server_version < 100000)
+        self.assertTrue(90000 <= server_version < 120000)
 
     def testAttributeStatus(self):
         status_ok = 1
@@ -270,19 +274,19 @@ class TestConnectObject(unittest.TestCase):
         self.assertRaises(TypeError, parameter)
         r = parameter('this server setting does not exist')
         self.assertIsNone(r)
-        s = query('show server_version').getresult()[0][0].upper()
+        s = query('show server_version').getresult()[0][0]
         self.assertIsNotNone(s)
         r = parameter('server_version')
         self.assertEqual(r, s)
-        s = query('show server_encoding').getresult()[0][0].upper()
+        s = query('show server_encoding').getresult()[0][0]
         self.assertIsNotNone(s)
         r = parameter('server_encoding')
         self.assertEqual(r, s)
-        s = query('show client_encoding').getresult()[0][0].upper()
+        s = query('show client_encoding').getresult()[0][0]
         self.assertIsNotNone(s)
         r = parameter('client_encoding')
         self.assertEqual(r, s)
-        s = query('show server_encoding').getresult()[0][0].upper()
+        s = query('show server_encoding').getresult()[0][0]
         self.assertIsNotNone(s)
         r = parameter('server_encoding')
         self.assertEqual(r, s)
@@ -406,6 +410,30 @@ class TestSimpleQueries(unittest.TestCase):
         self.assertEqual(v._fields, ('alias0',))
         self.assertEqual(v.alias0, 0)
 
+    def testNamedresultWithGoodFieldnames(self):
+        q = 'select 1 as snake_case_alias, 2 as "CamelCaseAlias"'
+        result = [(1, 2)]
+        r = self.c.query(q).namedresult()
+        self.assertEqual(r, result)
+        v = r[0]
+        self.assertEqual(v._fields, ('snake_case_alias', 'CamelCaseAlias'))
+
+    def testNamedresultWithBadFieldnames(self):
+        try:
+            r = namedtuple('Bad', ['?'] * 6, rename=True)
+        except TypeError:  # Python 2.6 or 3.0
+            fields = tuple('column_%d' % n for n in range(6))
+        else:
+            fields = r._fields
+        q = ('select 3 as "0alias", 4 as _alias, 5 as "alias$", 6 as "alias?",'
+            ' 7 as "kebap-case-alias", 8 as break, 9 as and_a_good_one')
+        result = [tuple(range(3, 10))]
+        r = self.c.query(q).namedresult()
+        self.assertEqual(r, result)
+        v = r[0]
+        self.assertEqual(v._fields[:6], fields)
+        self.assertEqual(v._fields[6], 'and_a_good_one')
+
     def testGet3Cols(self):
         q = "select 1,2,3"
         result = [(1, 2, 3)]
@@ -491,6 +519,7 @@ class TestSimpleQueries(unittest.TestCase):
             ' 0 as a_long_name_with_underscores,'
             ' 0 as "A long name with Blanks"')
         r = self.c.query(q).listfields()
+        self.assertIsInstance(r, tuple)
         result = ('a', 'b', 'c', 'c', 'b', 'a',
             'lowercase', 'uppercase', 'mixedcase', 'MixedCase',
             'a_long_name_with_underscores',
@@ -910,6 +939,7 @@ class TestQueryResultTypes(unittest.TestCase):
         self.c = connect()
         self.c.query('set client_encoding=utf8')
         self.c.query("set datestyle='ISO,YMD'")
+        self.c.query("set timezone='UTC'")
 
     def tearDown(self):
         self.c.close()
@@ -1802,6 +1832,31 @@ class TestConfigFunctions(unittest.TestCase):
         r = pg.get_namedresult()
         self.assertIs(r, namedresult)
 
+    def testSetRowFactorySize(self):
+        try:
+            from functools import lru_cache
+        except ImportError:  # Python < 3.2
+            lru_cache = None
+        queries = ['select 1 as a, 2 as b, 3 as c', 'select 123 as abc']
+        query = self.c.query
+        for maxsize in (None, 0, 1, 2, 3, 10, 1024):
+            pg.set_row_factory_size(maxsize)
+            for i in range(3):
+                for q in queries:
+                    r = query(q).namedresult()[0]
+                    if q.endswith('abc'):
+                        self.assertEqual(r, (123,))
+                        self.assertEqual(r._fields, ('abc',))
+                    else:
+                        self.assertEqual(r, (1, 2, 3))
+                        self.assertEqual(r._fields, ('a', 'b', 'c'))
+            if lru_cache:
+                info = pg._row_factory.cache_info()
+                self.assertEqual(info.maxsize, maxsize)
+                self.assertEqual(info.hits + info.misses, 6)
+                self.assertEqual(info.hits,
+                    0 if maxsize is not None and maxsize < 2 else 4)
+
 
 class TestStandaloneEscapeFunctions(unittest.TestCase):
     """Test pg escape functions.
@@ -1817,10 +1872,16 @@ class TestStandaloneEscapeFunctions(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
-        query = connect().query
+        db = connect()
+        query = db.query
         query('set client_encoding=sql_ascii')
         query('set standard_conforming_strings=off')
-        query('set bytea_output=escape')
+        try:
+            query('set bytea_output=escape')
+        except pg.ProgrammingError:
+            if db.server_version >= 90000:
+                raise  # ignore for older server versions
+        db.close()
         cls.cls_set_up = True
 
     def testEscapeString(self):
